<?php
/*
 Plugin Name: BuddyPress Restrict Group Creation
 Plugin URI: http://wordpress.org/extend/plugins/buddypress-restrict-group-creation/
 Description: Restrict group creation and settings to certain WP Role/Capabilities.
 Author: rich fuller - etiviti (rich!)
 Author URI: http://buddypress.org/developers/nuprn1/
 License: GNU GENERAL PUBLIC LICENSE 3.0 http://www.gnu.org/licenses/gpl.txt
 Version: 0.1.3
 Text Domain: bp-restrictgroups
 Site Wide Only: true
*/

//
// if you have checked 'demote group creator to mod' enabled
// You may auto-assign an user_id to all groups as group admin and/or group mod
//
//define( 'BP_RESTRICTGROUP_AUTOADD_ADMIN_USER_ID', 1 );
//define( 'BP_RESTRICTGROUP_AUTOADD_MOD_USER_ID', 2 );



//
// TODO'ing
//

//- ability to add restricted/censor group names via filter

//axe the steps
function bp_restrictgroups_create_group( ) {
	global $bp;
	
	//need to add the slug to the list
	$bp->groups->forbidden_names[] = 'closed';
	
	if ( is_site_admin() )
		return;
	
	if ( current_user_can( get_option('bp_restrictgroups_cap_low') ) )
		return;
	
	//clear current steps
	unset($bp->groups->group_creation_steps);
	
	$bp->groups->group_creation_steps['closed'] = array( 'name' => __( 'Closed', 'bp-restrictgroups' ), 'position' => 0 );
	
	add_action( 'groups_custom_create_steps', 'bp_restrictgroups_custom_create_step_restrict', 1 );
}
add_action( 'groups_setup_globals', 'bp_restrictgroups_create_group', 1000 );

//wait, what, why... the above is called before other extensions may hook into the group extension api and declare a 'step' - so just tidy it up
function bp_restrictgroups_create_group_steps( ) {
	global $bp;
	
	if ( is_site_admin() )
		return;
	
	if ( current_user_can( get_option('bp_restrictgroups_cap_low') ) )
		return;
	
	//clear current steps
	unset($bp->groups->group_creation_steps);
	$bp->groups->group_creation_steps['closed'] = array( 'name' => __( 'Closed', 'bp-restrictgroups' ), 'position' => 0 );
}
add_action( 'bp_before_create_group', 'bp_restrictgroups_create_group_steps', 1000 );

//give the user some feedback
function bp_restrictgroups_custom_create_step_restrict( ) {
	global $bp;
	
	if ( bp_is_group_creation_step( 'closed' ) ) : ?>
		<h4><?php _e('New Group creation is closed', 'bp-restrictgroups') ?></h4>
	<?php endif;
	
}

//kill the submit button
function bp_restrictgroups_insert_head() { 

	if ( is_site_admin() )
		return;

	if ( current_user_can( get_option('bp_restrictgroups_cap_low') ) )
		return;
		
	if (bp_is_groups_component() || bp_is_group_create() )
		echo '<style type="text/css">#previous-next { display:none; } form#groups-directory-form .button { display: none; }</style>';
}
add_action( 'wp_head', 'bp_restrictgroups_insert_head',1);

//add our new step to replace the group settings
function bp_restrictgroups_create_group_settings( ) {
	global $bp;
	
	//need to add the slug to the list
	$bp->groups->forbidden_names[] = 'group-settings-restricted';
	
	if ( is_site_admin() )
		return;
	
	//clear current steps
	unset($bp->groups->group_creation_steps['group-settings']);
	
	$bp->groups->group_creation_steps['group-settings-restricted'] = array( 'name' => __( 'Settings', 'buddypress' ), 'position' => 10 );
	
	add_action( 'groups_custom_create_steps', 'bp_restrictgroups_custom_create_step_settings', 1 );
	
}
add_action( 'groups_setup_globals', 'bp_restrictgroups_create_group_settings', 1000 );

//remove valid status types?
//function bp_queuegroups_edit_valid_status( $args ) {
//'public', 'private', 'hidden'
//}
//add_filter( 'groups_valid_status', 'bp_queuegroups_edit_valid_status', 1, 1);

//our new group settings page
function bp_restrictgroups_custom_create_step_settings( ) { ?>

	<?php if ( bp_is_group_creation_step( 'group-settings-restricted' ) ) : ?>

		<?php do_action( 'bp_before_group_settings_creation_step' ); ?>

		<?php if ( function_exists('bp_wire_install') ) : ?>
		<div class="checkbox">
			<label><input type="checkbox" name="group-show-wire" id="group-show-wire" value="1"<?php if ( bp_get_new_group_enable_wire() ) { ?> checked="checked"<?php } ?> /> <?php _e('Enable comment wire', 'buddypress') ?></label>
		</div>
		<?php endif; ?>

		<?php if ( function_exists('bp_forums_is_installed_correctly') ) : ?>
			<?php if ( bp_forums_is_installed_correctly() ) : ?>
				<?php if ( current_user_can( get_option('bp_restrictgroups_cap_enable_forum') ) ) : ?>
				<div class="checkbox">
					<label><input type="checkbox" name="group-show-forum" id="group-show-forum" value="1"<?php if ( bp_get_new_group_enable_forum() ) { ?> checked="checked"<?php } ?> /> <?php _e('Enable discussion forum', 'buddypress') ?></label>
				</div>
				<?php endif; ?>
			<?php else : ?>
				<?php if ( is_site_admin() ) : ?>
					<div class="checkbox">
						<label><input type="checkbox" disabled="disabled" name="disabled" id="disabled" value="0" /> <?php printf( __('<strong>Attention Site Admin:</strong> Group forums require the <a href="%s">correct setup and configuration</a> of a bbPress installation.', 'buddypress' ), bp_get_root_domain() . '/wp-admin/admin.php?page=bb-forums-setup' ) ?></label>
					</div>
				<?php endif; ?>
			<?php endif; ?>
		<?php endif; ?>

		<hr />

		<h4><?php _e( 'Privacy Options', 'buddypress' ); ?></h4>

		<div class="radio">

				<label><input type="radio" name="group-status" value="public"<?php if ( 'public' == bp_get_new_group_status() || !bp_get_new_group_status() ) { ?> checked="checked"<?php } ?> />
					<strong><?php _e( 'This is a public group', 'buddypress' ) ?></strong>
					<ul>
						<li><?php _e( 'Any site member can join this group.', 'buddypress' ) ?></li>
						<li><?php _e( 'This group will be listed in the groups directory and in search results.', 'buddypress' ) ?></li>
						<li><?php _e( 'Group content and activity will be visible to any site member.', 'buddypress' ) ?></li>
					</ul>
				</label>

				<?php if ( bp_restrictgroups_enabled_status( 'private' ) ) : ?>
					<label><input type="radio" name="group-status" value="private"<?php if ( 'private' == bp_get_new_group_status() ) { ?> checked="checked"<?php } ?> />
						<strong><?php _e( 'This is a private group', 'buddypress' ) ?></strong>
						<ul>
							<li><?php _e( 'Only users who request membership and are accepted can join the group.', 'buddypress' ) ?></li>
							<li><?php _e( 'This group will be listed in the groups directory and in search results.', 'buddypress' ) ?></li>
							<li><?php _e( 'Group content and activity will only be visible to members of the group.', 'buddypress' ) ?></li>
						</ul>
					</label>
				<?php endif; ?>
				
				<?php if ( bp_restrictgroups_enabled_status( 'hidden' ) ) : ?>
					<label><input type="radio" name="group-status" value="hidden"<?php if ( 'hidden' == bp_get_new_group_status() ) { ?> checked="checked"<?php } ?> />
						<strong><?php _e('This is a hidden group', 'buddypress') ?></strong>
						<ul>
							<li><?php _e( 'Only users who are invited can join the group.', 'buddypress' ) ?></li>
							<li><?php _e( 'This group will not be listed in the groups directory or search results.', 'buddypress' ) ?></li>
							<li><?php _e( 'Group content and activity will only be visible to members of the group.', 'buddypress' ) ?></li>
						</ul>
					</label>
				<?php endif; ?>
			
		</div>

		<?php do_action( 'bp_after_group_settings_creation_step' ); ?>

		<?php wp_nonce_field( 'groups_create_save_group-settings-restricted' ) ?>

	<?php endif; ?>

<?php }

//our new step_save action - also verify the data against caps
function bp_restrictgroups_groups_create_group_step_save( ) {
	global $bp;
	
	$group_status = 'public';
	$group_enable_forum = 1;

	if ( !isset($_POST['group-show-forum']) ) {
		$group_enable_forum = 0;
	} else {
		/* Create the forum if enable_forum = 1 */
		if ( function_exists( 'bp_forums_setup' ) && current_user_can( get_option('bp_restrictgroups_cap_enable_forum') ) && '' == groups_get_groupmeta( $bp->groups->new_group_id, 'forum_id' ) ) {
			groups_new_group_forum();
		}
	}

	if ( 'private' == $_POST['group-status'] && bp_restrictgroups_enabled_status( 'private' ))
		$group_status = 'private';
	else if ( 'hidden' == $_POST['group-status'] && bp_restrictgroups_enabled_status( 'hidden' ))
		$group_status = 'hidden';

	//we need to hook in the user_id change
	if (get_option('bp_restrictgroups_demote_creator_id')) add_action( 'groups_created_group', 'bp_restrictgroups_add_admin', 1, 1);

	if ( !$bp->groups->new_group_id = groups_create_group( array( 'group_id' => $bp->groups->new_group_id, 'status' => $group_status, 'enable_forum' => $group_enable_forum ) ) ) {
		bp_core_add_message( __( 'There was an error saving group details, please try again.', 'buddypress' ), 'error' );
		bp_core_redirect( $bp->root_domain . '/' . $bp->groups->slug . '/create/step/' . $bp->groups->current_create_step . '/' );
	}
	
}
add_action( 'groups_create_group_step_save_group-settings-restricted', 'bp_restrictgroups_groups_create_group_step_save', 1 );

//if a new group is created - lets always make sure our default admin is attached
function bp_restrictgroups_add_admin( $args ) {
	global $bp;
	
	if (!$args)
		return;
	
	if ( is_site_admin() )
		return;
	
	$add = get_option('bp_restrictgroups_demote_creator_id');
	if ($add) {

		$member = new BP_Groups_Member($bp->loggedin_user->id, $args);
		$member->is_admin = 0;
		$member->is_mod = 1;
		$member->user_title = __( 'Group Mod', 'buddypress' );
		$member->date_modified = gmdate( "Y-m-d H:i:s" );
		$member->save();

		if ( defined( 'BP_RESTRICTGROUP_AUTOADD_ADMIN_USER_ID' ) ) {
			$member = new BP_Groups_Member(BP_RESTRICTGROUP_AUTOADD_ADMIN_USER_ID, $args);
			$member->is_admin = 1;
			$member->is_mod = 0;
			$member->user_title = __( 'Group Admin', 'buddypress' );
			$member->is_confirmed = 1;
			$member->date_modified = gmdate( "Y-m-d H:i:s" );
			$member->save();
		}
		
		if ( defined( 'BP_RESTRICTGROUP_AUTOADD_MOD_USER_ID' ) ) {
			$member = new BP_Groups_Member(BP_RESTRICTGROUP_AUTOADD_MOD_USER_ID, $args);
			$member->is_admin = 0;
			$member->is_mod = 1;
			$member->user_title = __( 'Group Mod', 'buddypress' );
			$member->is_confirmed = 1;
			$member->date_modified = gmdate( "Y-m-d H:i:s" );
			$member->save();
		}
	}

}


//simple lookup function
function bp_restrictgroups_enabled_status( $key ) {

	$status = maybe_unserialize(get_option('bp_restrictgroups_cap_enable_status'));
	
	if ($status && $status[$key]) {
		if ( current_user_can( $status[$key] ) )
			return $status[$key];
	}

	return false;
}

//add admin_menu page
function bp_restrictgroups_add_admin_menu() {
	global $bp;

	if ( !is_site_admin() )
		return false;

	//Add the component's administration tab under the "BuddyPress" menu for site administrators
	require ( dirname( __FILE__ ) . '/bp-restrict-group-creation-admin.php' );

	add_submenu_page( 'bp-general-settings', __( 'Restrict Group Creation Admin', 'bp-restrictgroups' ), __( 'Restrict Group Creation', 'bp-restrictgroups' ), 'manage_options', 'bp-restrictgroups-settings', 'bp_restrictgroups_admin' );	

	//set up defaults
	add_option('bp_restrictgroups_cap_low', 'edit_posts' ); //contributor
	
	add_option('bp_restrictgroups_cap_enable_forum', 'upload_files' ); //author
	add_option('bp_restrictgroups_cap_enable_status', maybe_serialize(array( 'private' => 'upload_files', 'hidden' => 'upload_files' )) );

	add_option('bp_restrictgroups_demote_creator_id', true );
}

if ( defined( 'BP_VERSION' ) ) {
	add_action( 'admin_menu', 'bp_restrictgroups_add_admin_menu', 20 );
	add_filter('plugin_row_meta', 'bp_restrictgroups_add_admin_plugin_link', 10, 2);
} else {
	add_action( 'bp_init', 'bp_restrictgroups_init');
}

function bp_restrictgroups_init() {
	add_action( 'admin_menu', 'bp_restrictgroups_add_admin_menu', 20 );
	add_filter('plugin_row_meta', 'bp_restrictgroups_add_admin_plugin_link', 10, 2);
}



//settings link on plugin page (i like it so tphtphphphphph)
function bp_restrictgroups_add_admin_plugin_link($links, $file) {

	if ( !is_site_admin() )
		return false;

	$base = plugin_basename(__FILE__);
	if ($file == $base) {
		$links[] = '<a href="admin.php?page=bp-restrictgroups-settings">' . __('Settings','bp-restrictgroups') . '</a>';
	}
	return $links;
}

?>